# Mandelbrot Generator
from pygame.locals import *
from time import gmtime, strftime
import pygame
import random

# loop around the Mandelbrot function until the value is greater than 2
# or the maximum number of loops is reached

def count_loops(x,y,max_loops):

    z = complex(0,0)
    c = complex(x,y)
    for loops in range(max_loops):
        z = z**2+c
        if abs(z) > 2:
            return loops
    return 0

def main():

    # set the size of the window
    width= 1024
    height = 1024
    window = pygame.display.set_mode((width,height),DOUBLEBUF)

    # set the offset for window and a zoom_factor
    horizontal_offset = width/2+100
    vertical_offset = height/2
    zoom_factor = height/3

    max_loops = 25

    # set up an array of random colour values
    max_colours = 10
    colours = [[0 for i in range(3)] for j in range(max_colours)]

    for col_no in range(1,max_colours):
        colours[col_no][0]=random.randint(0,255)
        colours[col_no][1]=random.randint(0,255)
        colours[col_no][2]=random.randint(0,255)

    for vertical in range(int(height/2)+1):
        for horizontal in range(width):

            # calculate the actual co-ordinates of the point to check
            x = float(horizontal-horizontal_offset)/zoom_factor
            y = float(vertical-vertical_offset)/zoom_factor

            # count the number of loops for this point
            no_of_loops = count_loops(x,y,max_loops)

            # pick a colour depending on the number of loops
            col_no=no_of_loops % max_colours
            colour = (colours[col_no][0],colours[col_no][1],colours[col_no][2])

            # the set is symmetrical about the x axis so set the point
            # above and below the line
            window.set_at((horizontal, vertical), colour)
            window.set_at((horizontal, height-vertical), colour)

        # indent this line one tab to the left to only see the image at the end
        pygame.display.update()

    # save the image to disk as a png file
    filename = 'mandelbrot_'+strftime("%H%M%S")+'.png'
    pygame.image.save(window,filename)

main()

 
